import os
from pathlib import Path
from dotenv import load_dotenv

load_dotenv(dotenv_path=Path("config") / ".env", override=True)

#if os.environ.get("ENABLE_OTEL_AUTO_INSTRUMENT", "1") == "1":
#    from opentelemetry.instrumentation.auto_instrumentation.sitecustomize import initialize
#
#    print("load open telemetry.")
#    initialize()

__all__ = ["run"]


def run():
    import argparse
    import uvicorn
    from smartutils.app.const import AppKey

    parser = argparse.ArgumentParser(description="Run app/main")
    parser.add_argument("--host", type=str, default="0.0.0.0", help="Host to bind")
    parser.add_argument("--port", type=int, default=80, help="Port to bind")
    parser.add_argument("--reload", action="store_true", help="Enable auto-reload")
    parser.add_argument(
        "--app",
        type=str,
        default=AppKey.FASTAPI.value,
        choices=AppKey.list(),
        help=f"App type, choices: {AppKey.list()}",
    )
    args = parser.parse_args()

    uvicorn.run(
        f"smartutils.app.main.{args.app}:create_app",
        host=args.host,
        port=args.port,
        reload=args.reload,
        workers=1,
        factory=True,
        access_log=False,
    )
